@@ -82,7 +82,7 @@ use crate::Env;
 /// assert_eq!(modified_env.get("PATH").unwrap(), layer_dir.join("bin"));
 /// assert_eq!(modified_env.get("CPATH"), None); // None, because CPATH is only added during build
 /// ```
-#[derive(Eq, PartialEq, Debug)]
+#[derive(Eq, PartialEq, Debug, Clone)]
 pub struct LayerEnv {
     all: LayerEnvDelta,
     build: LayerEnvDelta,
@@ -98,9 +98,9 @@ pub struct LayerEnv {
 }
 
 impl LayerEnv {
-    /// Creates an empty LayerEnv that does not modify any environment variables.
+    /// Creates an empty `LayerEnv` that does not modify any environment variables.
     ///
-    /// Entries can be added with the [LayerEnv::insert] function.
+    /// Entries can be added with the [`LayerEnv::insert`] function.
     ///
     /// # Example:
     /// ```
@@ -114,7 +114,7 @@ impl LayerEnv {
     /// assert_eq!(env, modified_env);
     /// ```
     pub fn new() -> Self {
-        LayerEnv {
+        Self {
             all: LayerEnvDelta::new(),
             build: LayerEnvDelta::new(),
             launch: LayerEnvDelta::new(),
@@ -163,7 +163,7 @@ impl LayerEnv {
             .fold(env.clone(), |env, delta| delta.apply(&env))
     }
 
-    /// Insert a new entry into this LayerEnv.
+    /// Insert a new entry into this `LayerEnv`.
     ///
     /// Should there already be an entry for the same target lifecycle, modification behavior and
     /// name, it will be updated with the new given value.
@@ -185,6 +185,9 @@ impl LayerEnv {
     /// assert_eq!(modified_env.get("VAR").unwrap(), "hello");
     /// assert_eq!(modified_env.get("VAR2").unwrap(), "bar");
     /// ```
+    ///
+    /// See [`LayerEnv::chainable_insert`] that allows easy creation of inline `LayerEnv` values
+    /// without intermediate variables.
     pub fn insert(
         &mut self,
         target: TargetLifecycle,
@@ -207,6 +210,50 @@ impl LayerEnv {
         target_delta.insert(modification_behavior, name, value);
     }
 
+    /// Chainable version of [`LayerEnv::insert`].
+    ///
+    /// Allows easy creation of inline `LayerEnv` values without intermediate variables. See
+    /// [`LayerEnv::insert`] docs for details.
+    ///
+    /// # Example:
+    /// ```
+    /// use libcnb::layer_env::{LayerEnv, ModificationBehavior, TargetLifecycle};
+    /// use libcnb::Env;
+    ///
+    /// fn something_that_needs_layer_env(layer_env: LayerEnv) {
+    ///     let mut env = Env::new();
+    ///     let modified_env = layer_env.apply(TargetLifecycle::Build, &env);
+    ///     assert_eq!(modified_env.get("VAR").unwrap(), "hello");
+    ///     assert_eq!(modified_env.get("VAR2").unwrap(), "bar");
+    /// }
+    ///
+    /// something_that_needs_layer_env(
+    ///     LayerEnv::new()
+    ///         .chainable_insert(
+    ///             TargetLifecycle::All,
+    ///             ModificationBehavior::Default,
+    ///             "VAR",
+    ///             "hello",
+    ///         )
+    ///         .chainable_insert(
+    ///             TargetLifecycle::All,
+    ///             ModificationBehavior::Append,
+    ///             "VAR2",
+    ///             "bar",
+    ///         ),
+    /// );
+    /// ```
+    pub fn chainable_insert(
+        mut self,
+        target: TargetLifecycle,
+        modification_behavior: ModificationBehavior,
+        name: impl Into<OsString>,
+        value: impl Into<OsString>,
+    ) -> Self {
+        self.insert(target, modification_behavior, name, value);
+        self
+    }
+
     /// Constructs a `LayerEnv` based on the given layer directory.
     ///
     /// Follows the rules described in the Cloud Native Buildpacks specification and adds implicit
@@ -239,8 +286,8 @@ impl LayerEnv {
     /// assert_eq!(modified_env.get("PATH").unwrap(), layer_dir.join("bin"));
     /// assert_eq!(modified_env.get("ZERO_WING").unwrap(), "ALL_YOUR_BASE_ARE_BELONG_TO_US");
     /// ```
-    pub fn read_from_layer_dir(layer_dir: impl AsRef<Path>) -> Result<LayerEnv, std::io::Error> {
-        let mut result_layer_env = LayerEnv::new();
+    pub fn read_from_layer_dir(layer_dir: impl AsRef<Path>) -> Result<Self, std::io::Error> {
+        let mut result_layer_env = Self::new();
 
         let bin_path = layer_dir.as_ref().join("bin");
         let lib_path = layer_dir.as_ref().join("lib");
@@ -262,7 +309,7 @@ impl LayerEnv {
                 let target_delta = match target_lifecycle {
                     TargetLifecycle::Build => &mut result_layer_env.layer_paths_build,
                     TargetLifecycle::Launch => &mut result_layer_env.layer_paths_launch,
-                    _ => panic!("Unexpected TargetLifecycle in read_from_layer_dir implementation. This is a libcnb implementation error!"),
+                    _ => unreachable!("Unexpected TargetLifecycle in read_from_layer_dir implementation. This is a libcnb implementation error!"),
                 };
 
                 target_delta.insert(ModificationBehavior::Prepend, &name, &path);
@@ -338,7 +385,7 @@ impl Default for LayerEnv {
 
 /// Environment variable modification behavior.
 /// ([CNB spec: Environment Variable Modification Rules](https://github.com/buildpacks/spec/blob/main/buildpack.md#environment-variable-modification-rules))
-#[derive(Eq, PartialEq, Debug)]
+#[derive(Eq, PartialEq, Debug, Clone)]
 pub enum ModificationBehavior {
     Append,
     Default,
@@ -380,14 +427,14 @@ pub enum TargetLifecycle {
     Process(String),
 }
 
-#[derive(Eq, PartialEq, Debug)]
+#[derive(Eq, PartialEq, Debug, Clone)]
 struct LayerEnvDelta {
     entries: BTreeMap<(ModificationBehavior, OsString), OsString>,
 }
 
 impl LayerEnvDelta {
-    fn new() -> LayerEnvDelta {
-        LayerEnvDelta {
+    fn new() -> Self {
+        Self {
             entries: BTreeMap::new(),
         }
     }
@@ -429,7 +476,7 @@ impl LayerEnvDelta {
 
                     result_env.insert(&name, new_value);
                 }
-                _ => (),
+                ModificationBehavior::Delimiter => (),
             };
         }
 
@@ -447,14 +494,6 @@ impl LayerEnvDelta {
         let mut layer_env = Self::new();
 
         for dir_entry in fs::read_dir(path.as_ref())? {
-            let path = dir_entry?.path();
-
-            // Rely on the Rust standard library for splitting stem and extension. Since paths
-            // are not necessarily UTF-8 encoded, this is not as trivial as it might look like.
-            // Think twice before changing this.
-            let file_name_stem = path.file_stem();
-            let file_name_extension = path.extension();
-
             // The CNB spec explicitly states:
             //
             // > File contents MUST NOT be evaluated by a shell or otherwise modified before
@@ -467,15 +506,18 @@ impl LayerEnvDelta {
             // determined that it also treats the file contents as raw bytes.
             // See: https://github.com/buildpacks/lifecycle/blob/a7428a55c2a14d8a37e84285b95dc63192e3264e/env/env.go#L73-L106
             use std::os::unix::ffi::OsStringExt;
+            let path = dir_entry?.path();
             let file_contents = OsString::from_vec(fs::read(&path)?);
 
+            // Rely on the Rust standard library for splitting stem and extension. Since paths
+            // are not necessarily UTF-8 encoded, this is not as trivial as it might look like.
+            // Think twice before changing this.
+            let file_name_stem = path.file_stem();
+            let file_name_extension = path.extension();
+
             if let Some(file_name_stem) = file_name_stem {
                 let modification_behavior = match file_name_extension {
-                    None => {
-                        // TODO: This is different for CNB API versions > 0.5:
-                        // https://github.com/buildpacks/lifecycle/blob/a7428a55c2a14d8a37e84285b95dc63192e3264e/env/env.go#L66-L71
-                        Some(ModificationBehavior::Override)
-                    }
+                    None => Some(ModificationBehavior::Override),
                     Some(file_name_extension) => match file_name_extension.to_str() {
                         Some("append") => Some(ModificationBehavior::Append),
                         Some("default") => Some(ModificationBehavior::Default),
@@ -502,6 +544,8 @@ impl LayerEnvDelta {
     }
 
     fn write_to_env_dir(&self, path: impl AsRef<Path>) -> Result<(), std::io::Error> {
+        use std::os::unix::ffi::OsStrExt;
+
         if path.as_ref().exists() {
             // This is a possible race condition if the path is deleted between the check and
             // removal by this code. We accept this for now to keep it simple.
@@ -514,7 +558,7 @@ impl LayerEnvDelta {
             let file_extension = match modification_behavior {
                 ModificationBehavior::Append => ".append",
                 ModificationBehavior::Default => ".default",
-                ModificationBehavior::Delimiter => ".delimiter",
+                ModificationBehavior::Delimiter => ".delim",
                 ModificationBehavior::Override => ".override",
                 ModificationBehavior::Prepend => ".prepend",
             };
@@ -524,7 +568,6 @@ impl LayerEnvDelta {
 
             let file_path = path.as_ref().join(file_name);
 
-            use std::os::unix::ffi::OsStrExt;
             fs::write(file_path, &value.as_bytes())?;
         }
 
@@ -545,7 +588,7 @@ impl LayerEnvDelta {
 }
 
 #[cfg(test)]
-mod test {
+mod tests {
     use std::cmp::Ordering;
     use std::collections::HashMap;
     use std::fs;
@@ -557,9 +600,9 @@ mod test {
     use super::LayerEnvDelta;
 
     /// Direct port of a test from the reference lifecycle implementation:
-    /// See: https://github.com/buildpacks/lifecycle/blob/a7428a55c2a14d8a37e84285b95dc63192e3264e/env/env_test.go#L105-L154
+    /// See: <https://github.com/buildpacks/lifecycle/blob/a7428a55c2a14d8a37e84285b95dc63192e3264e/env/env_test.go#L105-L154>
     #[test]
-    fn test_reference_impl_env_files_have_a_suffix_it_performs_the_matching_action() {
+    fn reference_impl_env_files_have_a_suffix_it_performs_the_matching_action() {
         let temp_dir = tempdir().unwrap();
 
         let mut files = HashMap::new();
@@ -626,9 +669,9 @@ mod test {
     }
 
     /// Direct port of a test from the reference lifecycle implementation:
-    /// See: https://github.com/buildpacks/lifecycle/blob/a7428a55c2a14d8a37e84285b95dc63192e3264e/env/env_test.go#L188-L210
+    /// See: <https://github.com/buildpacks/lifecycle/blob/a7428a55c2a14d8a37e84285b95dc63192e3264e/env/env_test.go#L188-L210>
     #[test]
-    fn test_reference_impl_env_files_have_no_suffix_default_action_is_override() {
+    fn reference_impl_env_files_have_no_suffix_default_action_is_override() {
         let temp_dir = tempdir().unwrap();
 
         let mut files = HashMap::new();
@@ -662,9 +705,9 @@ mod test {
     }
 
     /// Direct port of a test from the reference lifecycle implementation:
-    /// See: https://github.com/buildpacks/lifecycle/blob/a7428a55c2a14d8a37e84285b95dc63192e3264e/env/env_test.go#L55-L80
+    /// See: <https://github.com/buildpacks/lifecycle/blob/a7428a55c2a14d8a37e84285b95dc63192e3264e/env/env_test.go#L55-L80>
     #[test]
-    fn test_reference_impl_add_root_dir_should_append_posix_directories() {
+    fn reference_impl_add_root_dir_should_append_posix_directories() {
         let temp_dir = tempdir().unwrap();
         fs::create_dir_all(temp_dir.path().join("bin")).unwrap();
         fs::create_dir_all(temp_dir.path().join("lib")).unwrap();
@@ -701,10 +744,25 @@ mod test {
     }
 
     #[test]
-    fn test_layer_env_delta_fs_read_write() {
+    fn layer_env_delta_fs_read_write() {
         let mut original_delta = LayerEnvDelta::new();
-        original_delta.insert(ModificationBehavior::Default, "FOO", "BAR");
         original_delta.insert(ModificationBehavior::Append, "APPEND_TO_ME", "NEW_VALUE");
+        original_delta.insert(
+            ModificationBehavior::Default,
+            "SET_THE_DEFAULT",
+            "DEFAULT_VALUE",
+        );
+        original_delta.insert(ModificationBehavior::Delimiter, "APPEND_TO_ME", ";");
+        original_delta.insert(
+            ModificationBehavior::Override,
+            "OVERRIDE_THIS",
+            "OVERRIDE_VALUE",
+        );
+        original_delta.insert(
+            ModificationBehavior::Prepend,
+            "PREPEND_THIS",
+            "PREPEND_VALUE",
+        );
 
         let temp_dir = tempdir().unwrap();
 
@@ -715,7 +773,7 @@ mod test {
     }
 
     #[test]
-    fn test_layer_env_insert() {
+    fn layer_env_insert() {
         let mut layer_env = LayerEnv::new();
         layer_env.insert(
             TargetLifecycle::Build,
@@ -756,7 +814,7 @@ mod test {
     }
 
     #[test]
-    fn test_modification_behavior_order() {
+    fn modification_behavior_order() {
         let tests = vec![
             (
                 ModificationBehavior::Append,
@@ -786,12 +844,12 @@ mod test {
         ];
 
         for (a, b, expected) in tests {
-            assert_eq!(expected, a.cmp(&b))
+            assert_eq!(expected, a.cmp(&b));
         }
     }
 
     #[test]
-    fn test_layer_env_delta_eq() {
+    fn layer_env_delta_eq() {
         let mut delta_1 = LayerEnvDelta::new();
         delta_1.insert(ModificationBehavior::Default, "a", "avalue");
         delta_1.insert(ModificationBehavior::Default, "b", "bvalue");
@@ -806,7 +864,7 @@ mod test {
     }
 
     #[test]
-    fn test_read_from_layer_dir_layer_paths_launch() {
+    fn read_from_layer_dir_layer_paths_launch() {
         let temp_dir = tempdir().unwrap();
         let layer_dir = temp_dir.path();
 
@@ -831,7 +889,7 @@ mod test {
     }
 
     #[test]
-    fn test_read_from_layer_dir_layer_paths_build() {
+    fn read_from_layer_dir_layer_paths_build() {
         let temp_dir = tempdir().unwrap();
         let layer_dir = temp_dir.path();
 
